@@ -8,16 +8,17 @@ use std::collections::HashMap;
 use std::fs::File;
 use std::io::{BufReader, BufWriter, Write};
 use std::sync::Arc;
-use vrp_cli::core::solver::population::{get_default_population, Population};
+use vrp_cli::core::solver::TargetHeuristic;
 use vrp_cli::extensions::solve::config::create_builder_from_config_file;
 use vrp_cli::scientific::tsplib::{TsplibProblem, TsplibSolution};
 use vrp_cli::{get_errors_serialized, get_locations_serialized};
-use vrp_core::models::problem::ObjectiveCost;
-use vrp_core::models::{Problem, Solution};
-use vrp_core::solver::hyper::*;
-use vrp_core::solver::population::{get_default_selection_size, Elitism};
-use vrp_core::solver::{Builder, Metrics, Telemetry, TelemetryMode};
-use vrp_core::utils::{DefaultRandom, Environment, Parallelism, Random};
+use vrp_core::construction::heuristics::InsertionContext;
+use vrp_core::models::problem::ProblemObjective;
+use vrp_core::prelude::*;
+use vrp_core::rosomaxa::evolution::*;
+use vrp_core::rosomaxa::{get_default_population, get_default_selection_size};
+use vrp_core::solver::*;
+use vrp_core::utils::{Parallelism, Quota, TimeQuota};
 
 const FORMAT_ARG_NAME: &str = "FORMAT";
 const PROBLEM_ARG_NAME: &str = "PROBLEM";
@@ -52,7 +53,7 @@ struct SolutionWriter(
             &Problem,
             Solution,
             f64,
-            Option<Metrics>,
+            Option<TelemetryMetrics>,
             BufWriter<Box<dyn Write>>,
             Option<BufWriter<Box<dyn Write>>>,
         ) -> Result<(), String>,
@@ -165,147 +166,147 @@ fn get_formats<'a>(matches: &ArgMatches, random: Arc<dyn Random + Send + Sync>)
     formats
 }
 
-pub fn get_solve_app<'a, 'b>() -> App<'a, 'b> {
+pub fn get_solve_app() -> App<'static> {
     App::new("solve")
         .about("Solves variations of Vehicle Routing Problem")
         .arg(
-            Arg::with_name(FORMAT_ARG_NAME)
+            Arg::new(FORMAT_ARG_NAME)
                 .help("Specifies the problem type")
                 .required(true)
                 .possible_values(&["solomon", "lilim", "tsplib", "pragmatic"])
                 .index(1),
         )
-        .arg(Arg::with_name(PROBLEM_ARG_NAME).help("Sets the problem file to use").required(true).index(2))
+        .arg(Arg::new(PROBLEM_ARG_NAME).help("Sets the problem file to use").required(true).index(2))
         .arg(
-            Arg::with_name(GENERATIONS_ARG_NAME)
+            Arg::new(GENERATIONS_ARG_NAME)
                 .help("Specifies maximum number of generations")
-                .short("n")
+                .short('n')
                 .long(GENERATIONS_ARG_NAME)
                 .required(false)
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(TIME_ARG_NAME)
+            Arg::new(TIME_ARG_NAME)
                 .help("Specifies max time algorithm run in seconds")
-                .short("t")
+                .short('t')
                 .long(TIME_ARG_NAME)
                 .required(false)
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(MIN_CV_ARG_NAME)
+            Arg::new(MIN_CV_ARG_NAME)
                 .help(
                     "Specifies variation coefficient termination criteria in form \"type,sample_size,threshold,is_global\"",
                 )
-                .short("v")
+                .short('v')
                 .long(MIN_CV_ARG_NAME)
                 .required(false)
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(INIT_SOLUTION_ARG_NAME)
+            Arg::new(INIT_SOLUTION_ARG_NAME)
                 .help("Specifies path to file with initial solution")
-                .short("i")
+                .short('i')
                 .long(INIT_SOLUTION_ARG_NAME)
                 .required(false)
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(INIT_SIZE_ARG_NAME)
+            Arg::new(INIT_SIZE_ARG_NAME)
                 .help("Specifies amount of initial solutions. Min is 1")
                 .long(INIT_SIZE_ARG_NAME)
                 .required(false)
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(MATRIX_ARG_NAME)
+            Arg::new(MATRIX_ARG_NAME)
                 .help("Specifies path to file with routing matrix")
-                .short("m")
+                .short('m')
                 .long(MATRIX_ARG_NAME)
-                .multiple(true)
+                .multiple_values(true)
                 .required(false)
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(OUT_RESULT_ARG_NAME)
+            Arg::new(OUT_RESULT_ARG_NAME)
                 .help("Specifies path to file for result output")
-                .short("o")
+                .short('o')
                 .long(OUT_RESULT_ARG_NAME)
                 .required(false)
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(GET_LOCATIONS_ARG_NAME)
+            Arg::new(GET_LOCATIONS_ARG_NAME)
                 .help("Returns list of unique locations")
-                .short("l")
+                .short('l')
                 .long(GET_LOCATIONS_ARG_NAME)
                 .required(false),
         )
         .arg(
-            Arg::with_name(GEO_JSON_ARG_NAME)
+            Arg::new(GEO_JSON_ARG_NAME)
                 .help("Specifies path to solution output in geo json format")
-                .short("g")
+                .short('g')
                 .long(GEO_JSON_ARG_NAME)
                 .required(false)
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(CONFIG_ARG_NAME)
+            Arg::new(CONFIG_ARG_NAME)
                 .help("Specifies path to algorithm configuration file")
-                .short("c")
+                .short('c')
                 .long(CONFIG_ARG_NAME)
                 .required(false)
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(LOG_ARG_NAME)
+            Arg::new(LOG_ARG_NAME)
                 .help("Specifies whether default logging is enabled")
                 .long(LOG_ARG_NAME)
                 .required(false)
                 .takes_value(false),
         )
         .arg(
-            Arg::with_name(CHECK_ARG_NAME)
+            Arg::new(CHECK_ARG_NAME)
                 .help("Specifies whether final solution should be checked for feasibility")
                 .long(CHECK_ARG_NAME)
                 .required(false)
                 .takes_value(false),
         )
         .arg(
-            Arg::with_name(SEARCH_MODE_ARG_NAME)
+            Arg::new(SEARCH_MODE_ARG_NAME)
                 .help("Specifies solution space search mode")
                 .long(SEARCH_MODE_ARG_NAME)
-                .short("s")
+                .short('s')
                 .required(false)
                 .possible_values(&["broad", "deep"])
                 .default_value("broad"),
         )
         .arg(
-            Arg::with_name(PARALELLISM_ARG_NAME)
+            Arg::new(PARALELLISM_ARG_NAME)
                 .help("Specifies data parallelism settings in format \"num_thread_pools,threads_per_pool\"")
                 .long(PARALELLISM_ARG_NAME)
-                .short("p")
+                .short('p')
                 .required(false)
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(HEURISTIC_ARG_NAME)
+            Arg::new(HEURISTIC_ARG_NAME)
                 .help("Specifies hyper heuristic algorithm")
                 .long(HEURISTIC_ARG_NAME)
-                .short("e")
+                .short('e')
                 .required(false)
                 .possible_values(&["default", "dynamic", "static"])
                 .default_value("default"),
         )
         .arg(
-            Arg::with_name(EXPERIMENTAL_ARG_NAME)
+            Arg::new(EXPERIMENTAL_ARG_NAME)
                 .help("Specifies whether experimental (unstable) features are enabled.")
                 .long(EXPERIMENTAL_ARG_NAME)
                 .required(false)
                 .takes_value(false),
         )
         .arg(
-            Arg::with_name(ROUNDED_ARG_NAME)
+            Arg::new(ROUNDED_ARG_NAME)
                 .help("Specifies whether costs are rounded. Applicable only for scientific formats.")
                 .long(ROUNDED_ARG_NAME)
                 .required(false)
@@ -318,7 +319,9 @@ pub fn run_solve(
     matches: &ArgMatches,
     out_writer_func: fn(Option<File>) -> BufWriter<Box<dyn Write>>,
 ) -> Result<(), String> {
-    let environment = get_environment(matches)?;
+    let max_time = parse_int_value::<usize>(matches, TIME_ARG_NAME, "max time")?;
+
+    let environment = get_environment(matches, max_time)?;
 
     let formats = get_formats(matches, environment.random.clone());
 
@@ -329,7 +332,6 @@ pub fn run_solve(
 
     // optional
     let max_generations = parse_int_value::<usize>(matches, GENERATIONS_ARG_NAME, "max generations")?;
-    let max_time = parse_int_value::<usize>(matches, TIME_ARG_NAME, "max time")?;
     let telemetry = Telemetry::new(if matches.is_present(LOG_ARG_NAME) {
         TelemetryMode::OnlyLogging {
             logger: environment.logger.clone(),
@@ -367,28 +369,41 @@ pub fn run_solve(
                             .map(|file| {
                                 init_reader.0(file, problem.clone())
                                     .map_err(|err| format!("cannot read initial solution '{}'", err))
-                                    .map(|solution| vec![solution])
+                                    .map(|solution| {
+                                        vec![InsertionContext::new_from_solution(
+                                            problem.clone(),
+                                            (solution, None),
+                                            environment.clone(),
+                                        )]
+                                    })
                             })
                             .unwrap_or_else(|| Ok(Vec::new()))?;
 
-                        let builder = if let Some(config) = config {
+                        let solver = if let Some(config) = config {
                             create_builder_from_config_file(problem.clone(), BufReader::new(config))
+                                .and_then(|builder| builder.build())
+                                .map(|config| Solver::new(problem.clone(), config))
                                 .map_err(|err| format!("cannot read config: '{}'", err))?
                         } else {
-                            Builder::new(problem.clone(), environment.clone())
+                            let config = create_default_config_builder(problem.clone(), environment.clone())
+                                .with_init_solutions(solutions, init_size)
                                 .with_telemetry(telemetry)
                                 .with_max_generations(max_generations)
                                 .with_max_time(max_time)
-                                .with_min_cv(min_cv)
-                                .with_population(get_population(mode, problem.objective.clone(), environment.clone()))
-                                .with_hyper(get_heuristic(matches, problem.clone(), environment)?)
+                                .with_min_cv(min_cv, "min_cv".to_string())
+                                .with_context(RefinementContext::new(
+                                    problem.clone(),
+                                    get_population(mode, problem.objective.clone(), environment.clone()),
+                                    environment.clone(),
+                                ))
+                                .with_heuristic(get_heuristic(matches, problem.clone(), environment)?)
+                                .build()?;
+
+                            Solver::new(problem.clone(), config)
                         };
 
-                        let (solution, cost, metrics) = builder
-                            .with_init_solutions(solutions, init_size)
-                            .build()
-                            .and_then(|solver| solver.solve())
-                            .map_err(|err| format!("cannot find any solution: '{}'", err))?;
+                        let (solution, cost, metrics) =
+                            solver.solve().map_err(|err| format!("cannot find any solution: '{}'", err))?;
 
                         solution_writer.0(&problem, solution, cost, metrics, out_buffer, geo_buffer).unwrap();
 
@@ -443,7 +458,9 @@ fn get_init_size(matches: &ArgMatches) -> Result<Option<usize>, String> {
         .unwrap_or(Ok(None))
 }
 
-fn get_environment(matches: &ArgMatches) -> Result<Arc<Environment>, String> {
+fn get_environment(matches: &ArgMatches, max_time: Option<usize>) -> Result<Arc<Environment>, String> {
+    let quota = max_time.map::<Arc<dyn Quota + Send + Sync>, _>(|time| Arc::new(TimeQuota::new(time as f64)));
+
     matches
         .value_of(PARALELLISM_ARG_NAME)
         .map(|arg| {
@@ -458,12 +475,18 @@ fn get_environment(matches: &ArgMatches) -> Result<Arc<Environment>, String> {
                 };
                 let is_experimental = matches.is_present(EXPERIMENTAL_ARG_NAME);
 
-                Ok(Arc::new(Environment::new(Arc::new(DefaultRandom::default()), parallelism, logger, is_experimental)))
+                Ok(Arc::new(Environment::new(
+                    Arc::new(DefaultRandom::default()),
+                    quota.clone(),
+                    parallelism,
+                    logger,
+                    is_experimental,
+                )))
             } else {
                 Err("cannot parse parallelism parameter".to_string())
             }
         })
-        .unwrap_or_else(|| Ok(Arc::new(Environment::default())))
+        .unwrap_or_else(|| Ok(Arc::new(Environment { quota, ..Environment::default() })))
 }
 
 fn get_matrix_files(matches: &ArgMatches) -> Option<Vec<File>> {
@@ -474,17 +497,17 @@ fn get_matrix_files(matches: &ArgMatches) -> Option<Vec<File>> {
 
 fn get_population(
     mode: Option<&str>,
-    objective: Arc<ObjectiveCost>,
+    objective: Arc<ProblemObjective>,
     environment: Arc<Environment>,
-) -> Box<dyn Population + Send + Sync> {
+) -> TargetPopulation {
     match mode {
-        Some("deep") => Box::new(Elitism::new(
+        Some("deep") => Box::new(ElitismPopulation::new(
             objective,
             environment.random.clone(),
             4,
             get_default_selection_size(environment.as_ref()),
         )),
-        _ => get_default_population(objective, environment),
+        _ => get_default_population::<RefinementContext, _, _>(objective, environment),
     }
 }
 
@@ -492,12 +515,12 @@ fn get_heuristic(
     matches: &ArgMatches,
     problem: Arc<Problem>,
     environment: Arc<Environment>,
-) -> Result<Box<dyn HyperHeuristic + Send + Sync>, String> {
+) -> Result<TargetHeuristic, String> {
     match matches.value_of(HEURISTIC_ARG_NAME) {
-        Some("dynamic") => Ok(Box::new(DynamicSelective::new_with_defaults(problem, environment))),
-        Some("static") => Ok(Box::new(StaticSelective::new_with_defaults(problem, environment))),
+        Some("dynamic") => Ok(get_dynamic_heuristic(problem, environment)),
+        Some("static") => Ok(get_static_heuristic(problem, environment)),
         Some(name) if name != "default" => Err(format!("unknown heuristic type name: '{}'", name)),
-        _ => Ok(Box::new(MultiSelective::new_with_defaults(problem, environment))),
+        _ => Ok(get_dynamic_heuristic(problem, environment)),
     }
 }
 
