@@ -11,14 +11,17 @@ extern crate serde_json;
 use serde::Deserialize;
 use std::io::{BufReader, Read};
 use std::sync::Arc;
+use vrp_core::construction::heuristics::InsertionContext;
 use vrp_core::models::common::SingleDimLoad;
-use vrp_core::models::Problem;
-use vrp_core::solver::hyper::*;
-use vrp_core::solver::mutation::*;
-use vrp_core::solver::population::*;
-use vrp_core::solver::processing::*;
-use vrp_core::solver::{Builder, Telemetry, TelemetryMode};
-use vrp_core::utils::{Environment, Parallelism, Random};
+use vrp_core::models::problem::ProblemObjective;
+use vrp_core::prelude::*;
+use vrp_core::rosomaxa::evolution::{InitialOperator, Telemetry, TelemetryMode};
+use vrp_core::rosomaxa::get_default_selection_size;
+use vrp_core::rosomaxa::prelude::*;
+use vrp_core::rosomaxa::utils::*;
+use vrp_core::solver::search::*;
+use vrp_core::solver::RecreateInitialOperator;
+use vrp_core::solver::*;
 
 /// An algorithm configuration.
 #[derive(Clone, Default, Deserialize, Debug)]
@@ -29,8 +32,6 @@ pub struct Config {
     pub hyper: Option<HyperType>,
     /// Specifies algorithm termination configuration.
     pub termination: Option<TerminationConfig>,
-    /// Specifies processing pipeline.
-    pub processing: Option<Vec<ProcessingType>>,
     /// Specifies environment configuration.
     pub environment: Option<EnvironmentConfig>,
     /// Specifies telemetry configuration.
@@ -127,23 +128,23 @@ pub enum SelectionType {
 #[derive(Clone, Deserialize, Debug)]
 #[serde(tag = "type")]
 pub enum HyperType {
-    /// A hyper heuristic which selects one mutation from the list based on its predefined probability.
+    /// A hyper heuristic which selects one operator from the list based on its predefined probability.
     #[serde(rename(deserialize = "static-selective"))]
     StaticSelective {
-        /// A collection of inner mutation operators (metaheuristics).
-        mutations: Option<Vec<MutationType>>,
+        /// A collection of inner operators (metaheuristics).
+        operators: Option<Vec<SearchOperatorType>>,
     },
 
-    /// A hyper heuristic which selects mutations from the predefined list using reinforcement
+    /// A hyper heuristic which selects operator from the predefined list using reinforcement
     /// learning technics.
     #[serde(rename(deserialize = "dynamic-selective"))]
     DynamicSelective,
 }
 
-/// A mutation operator configuration.
+/// A operator configuration.
 #[derive(Clone, Deserialize, Debug)]
 #[serde(tag = "type")]
-pub enum MutationType {
+pub enum SearchOperatorType {
     /// A metaheuristic which splits problem into smaller and solves them independently.
     #[serde(rename(deserialize = "decomposition"))]
     #[serde(rename_all = "camelCase")]
@@ -152,15 +153,15 @@ pub enum MutationType {
         routes: MinMaxConfig,
         /// Amount of attempts to repeat refinement.
         repeat: usize,
-        /// Probability of mutation.
-        probability: MutationProbabilityType,
+        /// Probability of operator.
+        probability: OperatorProbabilityType,
     },
 
     /// A local search heuristic.
     #[serde(rename(deserialize = "local-search"))]
     LocalSearch {
-        /// Probability of mutation.
-        probability: MutationProbabilityType,
+        /// Probability of operator.
+        probability: OperatorProbabilityType,
         /// Amount of times one of operators is applied.
         times: MinMaxConfig,
         /// Local search operator.
@@ -171,7 +172,7 @@ pub enum MutationType {
     #[serde(rename(deserialize = "ruin-recreate"))]
     RuinRecreate {
         /// Probability.
-        probability: MutationProbabilityType,
+        probability: OperatorProbabilityType,
         /// Ruin methods.
         ruins: Vec<RuinGroupConfig>,
         /// Recreate methods.
@@ -179,13 +180,13 @@ pub enum MutationType {
     },
 }
 
-/// A mutation method probability type
+/// A operator probability type
 #[derive(Clone, Deserialize, Debug)]
 #[serde(untagged)]
-pub enum MutationProbabilityType {
+pub enum OperatorProbabilityType {
     /// A scalar probability based type.
     Scalar {
-        /// Probability value of the mutation.
+        /// Probability value of the operator.
         scalar: f64,
     },
 
@@ -329,18 +330,6 @@ pub enum LocalOperatorType {
     Sequence { weight: usize },
 }
 
-/// A processing configuration type.
-#[derive(Clone, Deserialize, Debug)]
-#[serde(tag = "type")]
-pub enum ProcessingType {
-    #[serde(rename(deserialize = "vicinity-clustering"))]
-    VicinityClustering,
-    #[serde(rename(deserialize = "advance-departure"))]
-    AdvanceDeparture,
-    #[serde(rename(deserialize = "unassignment-reason"))]
-    UnassignmentReason,
-}
-
 #[derive(Clone, Deserialize, Debug)]
 pub struct NoiseConfig {
     probability: f64,
@@ -441,14 +430,16 @@ pub struct NameWeight {
 }
 
 fn configure_from_evolution(
-    mut builder: Builder,
+    mut builder: ProblemConfigBuilder,
+    problem: Arc<Problem>,
+    environment: Arc<Environment>,
     population_config: &Option<EvolutionConfig>,
-) -> Result<Builder, String> {
+) -> Result<ProblemConfigBuilder, String> {
     if let Some(config) = population_config {
         if let Some(initial) = &config.initial {
-            let environment = builder.config.environment.clone();
+            let environment = environment.clone();
 
-            builder = builder.with_init_params(
+            builder = builder.with_initial(
                 initial.alternatives.max_size,
                 initial.alternatives.quota,
                 std::iter::once(create_recreate_method(&initial.method, environment.clone()))
@@ -459,25 +450,37 @@ fn configure_from_evolution(
                             .iter()
                             .map(|method| create_recreate_method(method, environment.clone())),
                     )
+                    .map::<(
+                        Box<
+                            dyn InitialOperator<
+                                    Context = RefinementContext,
+                                    Objective = ProblemObjective,
+                                    Solution = InsertionContext,
+                                > + Send
+                                + Sync,
+                        >,
+                        _,
+                    ), _>(|(recreate, weight)| {
+                        (Box::new(RecreateInitialOperator::new(recreate)), weight)
+                    })
                     .collect(),
             );
         }
 
         if let Some(variation) = &config.population {
-            let default_selection_size = get_default_selection_size(builder.config.environment.as_ref());
+            let default_selection_size = get_default_selection_size(environment.as_ref());
             let population = match &variation {
-                PopulationType::Greedy { selection_size } => Box::new(Greedy::new(
-                    builder.config.problem.objective.clone(),
+                PopulationType::Greedy { selection_size } => Box::new(GreedyPopulation::new(
+                    problem.objective.clone(),
                     selection_size.unwrap_or(default_selection_size),
                     None,
                 )),
-                PopulationType::Elitism { max_size, selection_size } => Box::new(Elitism::new(
-                    builder.config.problem.objective.clone(),
-                    builder.config.environment.random.clone(),
+                PopulationType::Elitism { max_size, selection_size } => Box::new(ElitismPopulation::new(
+                    problem.objective.clone(),
+                    environment.random.clone(),
                     max_size.unwrap_or(4),
                     selection_size.unwrap_or(default_selection_size),
-                ))
-                    as Box<dyn Population + Send + Sync>,
+                )) as TargetPopulation,
                 PopulationType::Rosomaxa {
                     max_elite_size,
                     max_node_size,
@@ -522,52 +525,41 @@ fn configure_from_evolution(
                         config.exploration_ratio = *exploration_ratio;
                     }
 
-                    Box::new(Rosomaxa::new(
-                        builder.config.problem.objective.clone(),
-                        builder.config.environment.clone(),
-                        config,
-                    )?)
+                    Box::new(RosomaxaPopulation::new(problem.objective.clone(), environment.clone(), config)?)
                 }
             };
 
-            builder = builder.with_population(population);
+            builder = builder.with_context(RefinementContext::new(problem, population, environment));
         }
     }
 
     Ok(builder)
 }
 
-fn configure_from_hyper(mut builder: Builder, hyper_config: &Option<HyperType>) -> Result<Builder, String> {
+fn configure_from_hyper(
+    mut builder: ProblemConfigBuilder,
+    problem: Arc<Problem>,
+    environment: Arc<Environment>,
+    hyper_config: &Option<HyperType>,
+) -> Result<ProblemConfigBuilder, String> {
     if let Some(config) = hyper_config {
         match config {
-            HyperType::StaticSelective { mutations } => {
-                let static_selective = if let Some(mutations) = mutations {
-                    let mutation_group = mutations
+            HyperType::StaticSelective { operators } => {
+                let static_selective = if let Some(operators) = operators {
+                    let heuristic_group = operators
                         .iter()
-                        .map(|mutation| {
-                            create_mutation(
-                                builder.config.problem.clone(),
-                                builder.config.environment.clone(),
-                                mutation,
-                            )
-                        })
+                        .map(|operator| create_operator(problem.clone(), environment.clone(), operator))
                         .collect::<Result<Vec<_>, _>>()?;
-                    vrp_core::solver::hyper::StaticSelective::new(mutation_group)
+                    get_static_heuristic_from_heuristic_group(heuristic_group)
                 } else {
-                    vrp_core::solver::hyper::StaticSelective::new_with_defaults(
-                        builder.config.problem.clone(),
-                        builder.config.environment.clone(),
-                    )
+                    get_static_heuristic(problem.clone(), environment.clone())
                 };
 
-                builder = builder.with_hyper(Box::new(static_selective));
+                builder = builder.with_heuristic(static_selective);
             }
             HyperType::DynamicSelective => {
-                let dynamic_selective = vrp_core::solver::hyper::DynamicSelective::new_with_defaults(
-                    builder.config.problem.clone(),
-                    builder.config.environment.clone(),
-                );
-                builder = builder.with_hyper(Box::new(dynamic_selective));
+                let dynamic_selective = get_dynamic_heuristic(problem, environment);
+                builder = builder.with_heuristic(dynamic_selective);
             }
         }
     }
@@ -575,31 +567,15 @@ fn configure_from_hyper(mut builder: Builder, hyper_config: &Option<HyperType>)
     Ok(builder)
 }
 
-fn configure_from_termination(mut builder: Builder, termination_config: &Option<TerminationConfig>) -> Builder {
+fn configure_from_termination(
+    mut builder: ProblemConfigBuilder,
+    termination_config: &Option<TerminationConfig>,
+) -> ProblemConfigBuilder {
     if let Some(config) = termination_config {
-        builder = builder
-            .with_max_time(config.max_time)
-            .with_max_generations(config.max_generations)
-            .with_min_cv(config.variation.as_ref().map(|v| (v.interval_type.clone(), v.value, v.cv, v.is_global)));
-    }
-
-    builder
-}
-
-fn configure_from_processing(mut builder: Builder, processing_config: &Option<Vec<ProcessingType>>) -> Builder {
-    if let Some(config) = processing_config {
-        let processors = config
-            .iter()
-            .map::<Arc<dyn Processing + Send + Sync>, _>(|p_type| match p_type {
-                ProcessingType::VicinityClustering => Arc::new(VicinityClustering::default()),
-                ProcessingType::AdvanceDeparture => Arc::new(AdvanceDeparture::default()),
-                ProcessingType::UnassignmentReason => Arc::new(UnassignmentReason::default()),
-            })
-            .collect::<Vec<_>>();
-
-        if !processors.is_empty() {
-            builder = builder.with_processing(Some(Arc::new(CompositeProcessing::new(processors))));
-        }
+        builder = builder.with_max_time(config.max_time).with_max_generations(config.max_generations).with_min_cv(
+            config.variation.as_ref().map(|v| (v.interval_type.clone(), v.value, v.cv, v.is_global)),
+            "min_cv".to_string(),
+        );
     }
 
     builder
@@ -632,13 +608,13 @@ fn create_recreate_method(
     }
 }
 
-fn create_mutation(
+fn create_operator(
     problem: Arc<Problem>,
     environment: Arc<Environment>,
-    mutation: &MutationType,
-) -> Result<(Arc<dyn Mutation + Send + Sync>, MutationProbability), String> {
-    Ok(match mutation {
-        MutationType::RuinRecreate { probability, ruins, recreates } => {
+    operator: &SearchOperatorType,
+) -> Result<(TargetHeuristicOperator, TargetHeuristicProbability), String> {
+    Ok(match operator {
+        SearchOperatorType::RuinRecreate { probability, ruins, recreates } => {
             let ruin = Arc::new(WeightedRuin::new(
                 ruins.iter().map(|g| create_ruin_group(&problem, environment.clone(), g)).collect(),
             ));
@@ -647,14 +623,14 @@ fn create_mutation(
             ));
             (
                 Arc::new(RuinAndRecreate::new(ruin, recreate)),
-                create_mutation_probability(probability, environment.random.clone()),
+                create_operator_probability(probability, environment.random.clone()),
             )
         }
-        MutationType::LocalSearch { probability, times, operators: inners } => {
+        SearchOperatorType::LocalSearch { probability, times, operators: inners } => {
             let operator = create_local_search(times, inners, environment.random.clone());
-            (Arc::new(LocalSearch::new(operator)), create_mutation_probability(probability, environment.random.clone()))
+            (Arc::new(LocalSearch::new(operator)), create_operator_probability(probability, environment.random.clone()))
         }
-        MutationType::Decomposition { routes, repeat, probability } => {
+        SearchOperatorType::Decomposition { routes, repeat, probability } => {
             if *repeat < 1 {
                 return Err(format!("repeat must be greater than 1. Specified: {}", repeat));
             }
@@ -662,23 +638,22 @@ fn create_mutation(
                 return Err(format!("min routes must be greater than 2. Specified: {}", routes.min));
             }
 
-            let mutation =
-                vrp_core::solver::hyper::StaticSelective::create_default_mutation(problem, environment.clone());
+            let operator = create_default_heuristic_operator(problem, environment.clone());
             (
-                Arc::new(DecomposeSearch::new(mutation, (routes.min, routes.max), *repeat)),
-                create_mutation_probability(probability, environment.random.clone()),
+                Arc::new(DecomposeSearch::new(operator, (routes.min, routes.max), *repeat)),
+                create_operator_probability(probability, environment.random.clone()),
             )
         }
     })
 }
 
-fn create_mutation_probability(
-    probability: &MutationProbabilityType,
+fn create_operator_probability(
+    probability: &OperatorProbabilityType,
     random: Arc<dyn Random + Send + Sync>,
-) -> MutationProbability {
+) -> TargetHeuristicProbability {
     match probability {
-        MutationProbabilityType::Scalar { scalar } => create_scalar_mutation_probability(*scalar, random),
-        MutationProbabilityType::Context { threshold, phases } => create_context_mutation_probability(
+        OperatorProbabilityType::Scalar { scalar } => create_scalar_operator_probability(*scalar, random),
+        OperatorProbabilityType::Context { threshold, phases } => create_context_operator_probability(
             threshold.jobs,
             threshold.routes,
             phases
@@ -729,7 +704,7 @@ fn create_ruin_method(
             *probability,
         ),
         RuinMethod::CloseRoute { probability } => (Arc::new(CloseRouteRemoval::default()), *probability),
-        RuinMethod::RandomRuin { probability } => (StaticSelective::create_default_random_ruin(), *probability),
+        RuinMethod::RandomRuin { probability } => (create_default_random_ruin(), *probability),
     }
 }
 
@@ -758,7 +733,11 @@ fn create_local_search(
     Arc::new(CompositeLocalOperator::new(operators, times.min, times.max))
 }
 
-fn configure_from_telemetry(builder: Builder, telemetry_config: &Option<TelemetryConfig>) -> Builder {
+fn configure_from_telemetry(
+    builder: ProblemConfigBuilder,
+    environment: Arc<Environment>,
+    telemetry_config: &Option<TelemetryConfig>,
+) -> ProblemConfigBuilder {
     const LOG_BEST: usize = 100;
     const LOG_POPULATION: usize = 1000;
     const TRACK_POPULATION: usize = 1000;
@@ -769,7 +748,7 @@ fn configure_from_telemetry(builder: Builder, telemetry_config: &Option<Telemetr
 
     let create_progress = |log_best: &Option<usize>, log_population: &Option<usize>, dump_population: &Option<bool>| {
         TelemetryMode::OnlyLogging {
-            logger: builder.config.environment.logger.clone(),
+            logger: environment.logger.clone(),
             log_best: log_best.unwrap_or(LOG_BEST),
             log_population: log_population.unwrap_or(LOG_POPULATION),
             dump_population: dump_population.unwrap_or(false),
@@ -786,7 +765,7 @@ fn configure_from_telemetry(builder: Builder, telemetry_config: &Option<Telemetr
             Some(MetricsConfig { enabled: metrics_enabled, track_population }),
         )) => match (progress_enabled, metrics_enabled) {
             (true, true) => TelemetryMode::All {
-                logger: builder.config.environment.logger.clone(),
+                logger: environment.logger.clone(),
                 log_best: log_best.unwrap_or(LOG_BEST),
                 log_population: log_population.unwrap_or(LOG_POPULATION),
                 track_population: track_population.unwrap_or(TRACK_POPULATION),
@@ -802,8 +781,11 @@ fn configure_from_telemetry(builder: Builder, telemetry_config: &Option<Telemetr
     builder.with_telemetry(Telemetry::new(telemetry_mode))
 }
 
-fn configure_from_environment(environment_config: &Option<EnvironmentConfig>) -> Arc<Environment> {
-    let mut environment = Environment::default();
+fn configure_from_environment(
+    environment_config: &Option<EnvironmentConfig>,
+    max_time: Option<usize>,
+) -> Arc<Environment> {
+    let mut environment = Environment::new_with_time_quota(max_time);
 
     if let Some(parallelism) = environment_config.as_ref().and_then(|c| c.parallelism.as_ref()) {
         // TODO validate parameters
@@ -831,23 +813,26 @@ pub fn read_config<R: Read>(reader: BufReader<R>) -> Result<Config, String> {
 }
 
 /// Creates a solver `Builder` from config file.
-pub fn create_builder_from_config_file<R: Read>(
+pub fn create_builder_from_config_file<R>(
     problem: Arc<Problem>,
     reader: BufReader<R>,
-) -> Result<Builder, String> {
+) -> Result<ProblemConfigBuilder, String>
+where
+    R: Read,
+{
     read_config(reader).and_then(|config| create_builder_from_config(problem, &config))
 }
 
 /// Creates a solver `Builder` from config.
-pub fn create_builder_from_config(problem: Arc<Problem>, config: &Config) -> Result<Builder, String> {
-    let environment = configure_from_environment(&config.environment);
-    let mut builder = Builder::new(problem, environment);
+pub fn create_builder_from_config(problem: Arc<Problem>, config: &Config) -> Result<ProblemConfigBuilder, String> {
+    let environment =
+        configure_from_environment(&config.environment, config.termination.as_ref().and_then(|t| t.max_time));
+    let mut builder = create_default_config_builder(problem.clone(), environment.clone());
 
-    builder = configure_from_telemetry(builder, &config.telemetry);
-    builder = configure_from_evolution(builder, &config.evolution)?;
-    builder = configure_from_hyper(builder, &config.hyper)?;
+    builder = configure_from_telemetry(builder, environment.clone(), &config.telemetry);
+    builder = configure_from_evolution(builder, problem.clone(), environment.clone(), &config.evolution)?;
+    builder = configure_from_hyper(builder, problem, environment, &config.hyper)?;
     builder = configure_from_termination(builder, &config.termination);
-    builder = configure_from_processing(builder, &config.processing);
 
     Ok(builder)
 }
