@@ -1,6 +1,5 @@
 use super::*;
 use std::fs::File;
-use vrp_core::models::examples::create_example_problem;
 
 #[test]
 fn can_read_full_config() {
@@ -59,16 +58,16 @@ fn can_read_full_config() {
 
     let hyper_config = config.hyper.expect("cannot get hyper");
     match hyper_config {
-        HyperType::StaticSelective { mutations } => {
-            let mutations = mutations.expect("cannot get mutations");
-            assert_eq!(mutations.len(), 4);
-            match mutations.first().unwrap() {
-                MutationType::Decomposition { routes, repeat, probability } => {
+        HyperType::StaticSelective { operators } => {
+            let operators = operators.expect("cannot get operators");
+            assert_eq!(operators.len(), 4);
+            match operators.first().unwrap() {
+                SearchOperatorType::Decomposition { routes, repeat, probability } => {
                     assert_eq!(*repeat, 4);
                     assert_eq!(routes.min, 2);
                     assert_eq!(routes.max, 4);
                     match probability {
-                        MutationProbabilityType::Context { threshold, phases } => {
+                        OperatorProbabilityType::Context { threshold, phases } => {
                             assert_eq!(threshold.jobs, 300);
                             assert_eq!(threshold.routes, 10);
                             assert_eq!(phases.len(), 2);
@@ -79,8 +78,8 @@ fn can_read_full_config() {
                 _ => unreachable!(),
             }
 
-            match mutations.get(1).unwrap() {
-                MutationType::LocalSearch { probability, times, operators: inners } => {
+            match operators.get(1).unwrap() {
+                SearchOperatorType::LocalSearch { probability, times, operators: inners } => {
                     assert_eq!(as_scalar_probability(probability), 0.05);
                     assert_eq!(*times, MinMaxConfig { min: 1, max: 2 });
                     assert_eq!(inners.len(), 4);
@@ -88,8 +87,8 @@ fn can_read_full_config() {
                 _ => unreachable!(),
             }
 
-            match mutations.get(2).unwrap() {
-                MutationType::RuinRecreate { probability, ruins, recreates } => {
+            match operators.get(2).unwrap() {
+                SearchOperatorType::RuinRecreate { probability, ruins, recreates } => {
                     assert_eq!(as_scalar_probability(probability), 1.);
                     assert_eq!(ruins.len(), 6);
                     assert_eq!(recreates.len(), 12);
@@ -97,8 +96,8 @@ fn can_read_full_config() {
                 _ => unreachable!(),
             }
 
-            match mutations.last().unwrap() {
-                MutationType::LocalSearch { probability, times, operators: inners } => {
+            match operators.last().unwrap() {
+                SearchOperatorType::LocalSearch { probability, times, operators: inners } => {
                     assert_eq!(as_scalar_probability(probability), 0.01);
                     assert_eq!(*times, MinMaxConfig { min: 1, max: 2 });
                     assert_eq!(inners.len(), 4);
@@ -113,9 +112,6 @@ fn can_read_full_config() {
     assert_eq!(termination.max_time, Some(300));
     assert_eq!(termination.max_generations, Some(3000));
 
-    let processing = config.processing.expect("no processing config");
-    assert_eq!(processing.len(), 3);
-
     let environment = config.environment.expect("no environment config");
     assert_eq!(environment.is_experimental, Some(false));
 
@@ -128,25 +124,6 @@ fn can_read_full_config() {
     assert_eq!(logging.prefix, Some("[config.full]".to_string()));
 }
 
-#[test]
-fn can_create_builder_from_config() {
-    let file = File::open("../examples/data/config/config.full.json").expect("cannot read config from file");
-    let config = read_config(BufReader::new(file)).unwrap();
-    let problem = create_example_problem();
-
-    let builder = create_builder_from_config(problem.clone(), &config).unwrap();
-
-    assert!(builder.config.population.variation.is_some());
-    assert_eq!(builder.config.problem.as_ref() as *const Problem, problem.as_ref() as *const Problem);
-    assert_eq!(builder.config.population.initial.max_size, 4);
-    assert_eq!(builder.config.population.initial.quota, 0.05);
-    assert_eq!(builder.config.population.initial.individuals.len(), 0);
-    assert_eq!(builder.config.population.initial.methods.len(), 8);
-    assert_eq!(builder.max_time, Some(300));
-    assert_eq!(builder.max_generations, Some(3000));
-    assert_eq!(builder.config.environment.is_experimental, false);
-}
-
 #[test]
 fn can_create_default_config() {
     let config = Config::default();
@@ -157,9 +134,9 @@ fn can_create_default_config() {
     assert!(config.telemetry.is_none());
 }
 
-fn as_scalar_probability(probability: &MutationProbabilityType) -> f64 {
+fn as_scalar_probability(probability: &OperatorProbabilityType) -> f64 {
     match probability {
-        MutationProbabilityType::Scalar { scalar } => *scalar,
+        OperatorProbabilityType::Scalar { scalar } => *scalar,
         _ => unreachable!(),
     }
 }
